; Printing CROSSTAB Tables Using a Pseudo 2-Dimensional Dynarray
;
; Karl Thompson -- Pine Grove Software
;
; This script will run in the PDOX40\VIDEO directory if:
; 1) you have used Paradox's default installation
; 2) you created a Sales table per the steps in last month's
;    article.
; 3) you created from SALES.DB a Crosstab table and renamed it
;    SALES_2.DB
 
; --------------------------------------------------------------
; This is the main procedure that calls the 2-D initialization
; procedure and the printing procedure
; --------------------------------------------------------------
PROC Main2DProc()
PRIVATE ProcName,
        aTableName,
        nNumCols,
        nNumFields,
        nFieldCounter,
        nFrstFieldNum,
        nDataArraySize,
        nLastElement,
        nIndx,
        nIndxAdjust,
        nNumLabels,
        nNumRecs

  ProcName = "Main2DProc"
  RESET

  ; Sales_2.db contains records for every month in Sales.db
  aTableName = "sales_2"  ; Table created with CROSSTAB command
  ; Use line below to print report with '91 sales records only
  ;aTableName = "sales_91"

  VIEW aTableName
  DYNARRAY aDataArray[]
  nNumRecs = NRECORDS(aTableName)
  ; The values are set for a typical report printed in
  ; compressed landscape mode
  ; Number of columns to print across includes number of
  ; columns added as row labels
  nNumCols = 11

  ; Use next line if the report is to print in compressed
  ; portrait mode (Try it both ways to demonstrate flexibility
  ; of algorithm)
  ;nNumCols = 8              ;don't forget to change comments in Prnt2DArray()
  ; nNumFields initialized here to avoid two calls to NFIELDS()
  nNumFields = NFIELDS(aTableName)

  nColCounter = 1
  nFieldCounter = 4  ; Initialize to the first column of data
                     ; beyond the column(s) of row labels
  nNumLabels = 3     ; Number of columns of row labels
                     ; (see notes elsewhere in code!!)
  nIndx = 0          ; Initial value of 1st dimension of dynarray
  nIndxAdjust = 100  ; 1st dimension adjusted by
                     ; This example will accommodate a Crosstab
                     ; table with up to 100 columns
  Init2DArray(aTableName,aDataArray,
              nNumFields,nIndx,nIndxAdjust)
  Prnt2DArray(aDataArray,nNumCols,nNumFields,nFieldCounter,
              nIndx,nIndxAdjust,nNumLabels)
  CLEARIMAGE
ENDPROC

; --------------------------------------------------------------
; Initializes a pseudo 2-dimensional dynarray
; --------------------------------------------------------------
PROC Init2DArray(aTableName,
                 aDataArray,
                 nNumFields,
                 nIndx,
                 nIndxAdjust)
PRIVATE ProcName,
        nSize,
        nOffSet,
        Indx2,
        Y,       ; Temp dynarray
        X        ; Counter

  ProcName = "Init2DArray"
  MESSAGE "Building Data Dynarray..."
  nFirstRow = nIndx

  ; Get Field Names for column headers
  FOR nOffSet FROM 1 TO nNumFields
    RIGHT
    aDataArray[nIndx+nOffSet] = Field()
    ; Modify a field name to fit column
    ; In this case change "Film #" to "Film"
    IF (nOffSet = 1) THEN
      aDataArray[nIndx+nOffSet] = "Film"
    ENDIF
    ; Here change "Category" to "Cat"
    IF (nOffSet = 2) THEN
      aDataArray[nIndx+nOffSet] = "Cat"
    ENDIF
  ENDFOR

  ; Initialize dynarray with first Crosstab table record 
  COPYTOARRAY Y
  nIndx = nIndx+nIndxAdjust
  FOR nOffSet FROM 1 TO nNumFields
    aDataArray[nIndx+nOffSet] = Y[nOffSet+1]
  ENDFOR

  ; Initialize dynarray with balance Crosstab table records 
  FOR X FROM 2 TO nNumRecs
    DOWN
    COPYTOARRAY Y
    nIndx = nIndx+nIndxAdjust
    FOR nOffSet FROM 1 TO nNumFields
      aDataArray[nIndx+nOffSet] = Y[nOffSet+1]
    ENDFOR
  ENDFOR

  ; Dynamic array is now initialized
  ; Any element can be accessed via aDataArray[nIndx+nOffSet]
  ; Thus we have a pseudo 2 dimensional dynarray
ENDPROC

; ------------------------------------------------------------
; Prints a the pseudo 2-dimensional dynarray
; ------------------------------------------------------------
PROC Prnt2DArray(aDataArray,
                 nNumCols,
                 nNumFields,
                 nFieldCounter,
                 nIndx,
                 nIndxAdjust,
                 nNumLabels)
PRIVATE ProcName,
        nFrstFieldNum,
        nDataArraySize,
        nLastElement,
        nColCounter,
        nOffSet,
        indx2,
        X

  ProcName = "Prnt2DArray"
  MESSAGE "Printing Report..."

  ; nFrstFieldNum serves as place holder for 1st column of data
  nFrstFieldNum = nFieldCounter
  X = nFieldCounter
  nDataArraySize = DYNARRAYSIZE(aDataArray)
  nLastElement = (nDataArraySize/nNumFields)*nIndxAdjust
  nColCounter = 1     ; First column being printed
  OPEN PRINTER
  PRINTER ON

  ; HP's LaserJet landscape lineprinter font values are setup
  ; for about 176 character columns on a page
  PRINT "\027&l1O\027(s16.66H"
  ;PRINT "\027(s16.66H"
  ; Optional report header goes here below line for landscape or
  ; comment out if portrait printing
  PRINT "                                   " +
        "                                   " +
  ; Use this line for portrait; comment out for landscape
  ;PRINT "                                              " +
         "ͻ"
  PRINT "\n"
  ; Use this line for landscape; comment out for portrait 
  PRINT "                                               " +
        "                       " +
  ; Use this line for portrait; comment out for landscape
  ;PRINT "                                              " +
         "  Number of Days a Film is Rented Each Month  "
  PRINT "\n"
  ; Use this line for landscape; comment out for portrait
  PRINT "                                               " +
        "                       " +
  ; Use this line for portrait; comment out for landscape
  ;PRINT "                                              " +
         "ͼ"
  PRINT "\n"
  PRINT "\n"
  ; End optional report header

  ; Outer-most WHILE loop controls printing of all fields. When
  ; X = nNumFields then last column of CROSSTAB is being printed
  WHILE X <= nNumFields
    ; 2nd WHILE loop controls the printing of rows. When
    ; nIndx = nLastElement then the last record is being printed
    WHILE nIndx < nLastElement
      ; Number of calls to PrntRowLabels must
      ; equal value of nNumLabels
      PrntRowLabels(aDataArray,nIndx,1,"W6,AL")   ; 1st column
      PrntRowLabels(aDataArray,nIndx,2,"W4,AL")   ; 2nd column
      PrntRowLabels(aDataArray,nIndx,3,"W32,AL")  ; 3rd column
      ; 3rd WHILE loop controls wrapping of records down page
      ; When nColCounter+nNumLabels=nNumCols printing last value
      ; that fits in row
      WHILE nColCounter + nNumLabels <= nNumCols
        ; Dynarray element will not be assigned if there are not
        ; enough dynarray elements to print to fill a row
        IF nFieldCounter <= nNumFields THEN
          PRINT FORMAT("W15,AR,EC,SP",
                       aDataArray[nIndx+nFieldCounter])
          nFieldCounter = nFieldCounter + 1
        ENDIF
        nColCounter = nColCounter + 1
      ENDWHILE

      ; Increase nIndx to "move" to next row of data
      nIndx = nIndx + nIndxAdjust
      nColCounter = 1
      X = nFieldCounter
      nFieldCounter = nFrstFieldNum

      ; Lines below would underline the last row before
      ; printing column totals
      ;IF (nIndx = nNumRecs) THEN
      ;  PRINT "\r"
      ;  PRINT "       "      ; indent w/o underline
      ;  PRINT Fill("_",165)  ; 165 is number of columns
      ;                       ; of compressed landscape
      ;  PRINT "\n"
      ;ENDIF
      PRINT "\n"
    ENDWHILE
    nFieldCounter = X
    nFrstFieldNum = X
    ; "Move" back to first record to print other fields that
    ; will not fit across page
    nIndx = 0
    PRINT "\n"
  ENDWHILE

  ; Optional Report Footer/Legend/Criteria here
  PRINT "Film = Film #       Cat = Film Category"
  PRINT "\027E"        ; HP LaserJet reset code
  CLOSE PRINTER
ENDPROC

; --------------------------------------------------------------
; Print the row labels
; Select a column (nFieldCounter) containing data for row label
; --------------------------------------------------------------
PROC PrntRowLabels(aDataArray,nOffSet,nFieldCounter,FormatStr)
PRIVATE ProcName
  ProcName = "PrntRowLabels"
  PRINT FORMAT(FormatStr,aDataArray[nOffSet+nFieldCounter])
ENDPROC

Main2DProc()
